"""
rare-earth metal and abundance got from wiki (https://en.wikipedia.org/wiki/Rare-earth_element)
"""

EXPENSIVE_METALS = {
                    "Au",  # Gold
                    "Ag",  # Silver
                    "Dy",  # Dysprosium
                    "Eu",  # Europium
                    "Ga",  # Gallium
                    "Gd",  # Gadolinium
                    "Hf",  # Hafnium
                    "In",  # Indium
                    "Ir",  # Iridium
                    "La",  # Lanthanum
                    "Mo",  # Molybdenum
                    "Nd",  # Neodymium
                    "Pd",  # Palladium
                    "Pr",  # Praseodymium
                    "Pt",  # Platinum
                    "Rh",  # Rhodium
                    "Ru",  # Ruthenium
                    "Se",  # Selenium
                    "Sm",  # Samarium
                    "Tb",  # Terbium
                    "Te",  # Tellurium
                    "Tm",  # Thulium
                    "U",   # Uranium
                    "Y"    # Yttrium
                }

RARE_METALS = {
                "Sc": 22,  # Scandium, 22 ppm
                "Y": 33,   # Yttrium, 33 ppm
                "La": 39,  # Lanthanum, 39 ppm
                "Ce": 66.5,  # Cerium, 66.5 ppm
                "Pr": 9.2,  # Praseodymium, 9.2 ppm
                "Nd": 41.5,  # Neodymium, 41.5 ppm
                "Pm": 10e-15,  # Promethium, 10e-15 ppm
                "Sm": 7.05,  # Samarium, 7.05 ppm
                "Eu": 2,  # Europium, 2 ppm
                "Gd": 6.2,  # Gadolinium, 6.2 ppm
                "Tb": 1.2,  # Terbium, 1.2 ppm
                "Dy": 5.2,  # Dysprosium, 5.2 ppm
                "Ho": 1.3,  # Holmium, 1.3 ppm
                "Er": 3.5,  # Erbium, 3.5 ppm
                "Tm": 0.52,  # Thulium, 0.52 ppm
                "Yb": 3.2,  # Ytterbium, 3.2 ppm
                "Lu":0.8   # Lutetium, 0.8 ppm
            }
