cv_features_minimal = ['site Number',
        'site AtomicWeight',
        'site Row',
        'site Column',
        'site Electronegativity',
        'site CovalentRadius',
        'AGNI eta=8.00e-01',
        'AGNI eta=1.23e+00',
        'AGNI eta=1.88e+00',
        'AGNI eta=2.89e+00',
        'AGNI eta=4.43e+00',
        'AGNI eta=6.80e+00',
        'AGNI eta=1.04e+01',
        'AGNI eta=1.60e+01',
        'G2_0.05',
        'G2_4.0',
        'G2_20.0',
        'G2_80.0',
        'G4_0.005_1.0_1.0',
        'G4_0.005_1.0_-1.0',
        'G4_0.005_4.0_1.0',
        'G4_0.005_4.0_-1.0',
        ]
cv_features= ['site Number',
        'site AtomicWeight',
        'site Row',
        'site Column',
        'site Electronegativity',
        'site CovalentRadius',
        'AGNI eta=8.00e-01',
        'AGNI eta=1.23e+00',
        'AGNI eta=1.88e+00',
        'AGNI eta=2.89e+00',
        'AGNI eta=4.43e+00',
        'AGNI eta=6.80e+00',
        'AGNI eta=1.04e+01',
        'AGNI eta=1.60e+01',
        'G2_0.05',
        'G2_4.0',
        'G2_20.0',
        'G2_80.0',
        'G4_0.005_1.0_1.0',
        'G4_0.005_1.0_-1.0',
        'G4_0.005_4.0_1.0',
        'G4_0.005_4.0_-1.0',
        'local difference in Number',
        'local difference in AtomicWeight',
        'local difference in Row',
        'local difference in Column',
        'local difference in Electronegativity',
        'local difference in CovalentRadius',
        'local signed difference in Number',
        'local signed difference in AtomicWeight',
        'local signed difference in Row',
        'local signed difference in Column',
        'local signed difference in Electronegativity',
        'local signed difference in CovalentRadius',
        'maximum local difference in Number',
        'maximum local difference in AtomicWeight',
        'maximum local difference in Row',
        'maximum local difference in Column',
        'maximum local difference in Electronegativity',
        'maximum local difference in CovalentRadius',
        'minimum local difference in Number',
        'minimum local difference in AtomicWeight',
        'minimum local difference in Row',
        'minimum local difference in Column',
        'minimum local difference in Electronegativity',
        'minimum local difference in CovalentRadius' 
        ]
